`timescale 1ns/10ps
`define CYCLE      25.0          	  // Modify your clock period here
`define End_CYCLE  13                 // Modify cycle times once your design need more cycle times!  
`include "sq_detector.v"
module testbench;
    
    // declare the needed regs and wires (the right hand side varibles inside the initail and always block)
    reg clk,reset;
    reg in_data;
    reg mem_i [9:0];
    reg golden [9:0];
    reg mem_o [9:0];
    wire out_data;
    // integer is for collecting the information which is irrelevant with hardwares
    integer i;
    integer error;
    
    // the hardware you want to verification
    SQD sq(
        .clk(clk),
        .rst(reset),
        .in_data(in_data),
        .out_data(out_data)
    );

    always begin #(`CYCLE/2) clk = ~clk; end // let the clk be opposite every half cycle

    // every initial block start with the time zero

    // this initial block do some preperation, such as generating vcd file, read the input file, print some information
    initial begin
        $dumpfile("trafficlight.vcd");                                          // time = 0
        $dumpvars(0,testbench);                                                 // time = 0
        $readmemh("../others/input.dat",mem_i);                                 // time = 0
        $readmemh("../others/golden.dat",golden);                               // time = 0
        $display("-----------------------------------------------------");      // time = 0
        $display("START!!! Simulation Start .....");                            // time = 0
        $display("-----------------------------------------------------");      // time = 0
    end

    // this initial block initiate the value of some signals
	initial begin
        clk = 1'b1;                         // time = 0
        reset = 1'b1;                       // time = 0
        in_data = 1'b0;                     // time = 0
        error = 0;                          // time = 0
        #(`CYCLE) reset = 1'b0;             // time = a cycle
        #(`CYCLE*`End_CYCLE) $finish;       // time = a cycle(above) + End_CYCLE*CYCLE
    end

    // this initial block run the task(case1)
    initial begin
        case1;      //start at time = 0;
    end


    // task block ack like a function in software, it will run squentially inside the block
    task case1; begin
        wait(reset==0);             // stall here untill reset become '0' (correspond to line 47)
        for(i=0;i<10;i++) begin
            in_data = mem_i[i];
            # 0.01;                 // wait for a moment for taking wanted value (if not will get the pre-cycle's value)
            if(out_data==golden[i]) begin
                $display("input= %d, out data equal to %d",in_data,golden[i]);
            end
            else begin
                $display("input= %d, out data is %d but golden is %d",in_data,out_data,golden[i]);
                error = 1;
            end
            @(posedge clk);         // stall here until the next clock posedge
        end
        if(error) begin
            $display("-----------------------------------------------------");
            $display("Simulation END, there are some errors.....");
            $display("-----------------------------------------------------");
        end
        else begin
            $display("-----------------------------------------------------");
            $display("Simulation END, SUCCESS!!!");
            $display("-----------------------------------------------------");
        end
    end endtask

endmodule
    